from enum import Enum

from checkout_sdk.common.common import Phone, Address
from checkout_sdk.common.enums import Currency, ChallengeIndicator


class ChannelType(str, Enum):
    BROWSER = 'browser'
    APP = 'app'


class SdkInterfaceType(str, Enum):
    NATIVE = 'native'
    HTML = 'html'
    BOTH = 'both'


class ThreeDsMethodCompletion(str, Enum):
    Y = 'y'
    N = 'n'
    U = 'u'


class CompletionInfoType(str, Enum):
    HOSTED = 'hosted'
    NON_HOSTED = 'non_hosted'


class SessionSourceType(str, Enum):
    CARD = 'card'
    ID = 'id'
    TOKEN = 'token'
    NETWORK_TOKEN = 'network_token'


class AuthenticationType(str, Enum):
    REGULAR = 'regular'
    RECURRING = 'recurring'


class Category(str, Enum):
    PAYMENT = 'payment'
    NON_PAYMENT = 'nonPayment'


class TransactionType(str, Enum):
    GOODS_SERVICE = 'goods_service'
    CHECK_ACCEPTANCE = 'check_acceptance'
    ACCOUNT_FUNDING = 'account_funding'
    QUASHI_CARD_TRANSACTION = 'quashi_card_transaction'
    PREPAID_ACTIVATION_AND_LOAD = 'prepaid_activation_and_load'


class UIElements(str, Enum):
    TEXT = 'text'
    SINGLE_SELECT = 'single_select'
    MULTI_SELECT = 'multi_select'
    OOB = 'oob'
    HTML_OTHER = 'html_other'


class SessionScheme(str, Enum):
    VISA = 'visa'
    MASTERCARD = 'mastercard'
    JCB = 'jcb'
    AMEX = 'amex'
    DINERS = 'diners'
    CARTES_BANCAIRES = 'cartes_bancaires'


class AuthenticationMethod(str, Enum):
    NO_AUTHENTICATION = 'no_authentication'
    OWN_CREDENTIALS = 'own_credentials'
    FEDERATED_ID = 'federated_id'
    ISSUER_CREDENTIALS = 'issuer_credentials'
    THIRD_PARTY_AUTHENTICATION = 'third_party_authentication'
    FIDO = 'fido'


class DeliveryTimeframe(str, Enum):
    ELECTRONIC_DELIVERY = 'electronic_delivery'
    SAME_DAY = 'same_day'
    OVERNIGHT = 'overnight'
    TWO_DAY_OR_MORE = 'two_day_or_more'


class ShippingIndicator(str, Enum):
    VISA = 'visa'


class SdkEphemeralPublicKey:
    kty: str
    crv: str
    x: str
    y: str


class SessionAddress(Address):
    address_line3: str


class SessionMarketplaceData:
    sub_entity_id: str


class SessionsBillingDescriptor:
    name: str


# Channel
class ChannelData:
    channel: ChannelType

    def __init__(self, channel_p: ChannelType):
        self.channel = channel_p


class AppSession(ChannelData):
    sdk_app_id: str
    sdk_max_timeout: int
    sdk_ephem_pub_key: SdkEphemeralPublicKey
    sdk_reference_number: str
    sdk_encrypted_data: str
    sdk_transaction_id: str
    sdk_interface_type: SdkInterfaceType
    sdk_ui_elements: list  # UIElements

    def __init__(self):
        super().__init__(ChannelType.APP)


class BrowserSession(ChannelData):
    three_ds_method_completion: ThreeDsMethodCompletion
    accept_header: str
    java_enabled: bool
    language: str
    color_depth: str
    screen_height: str
    screen_width: str
    timezone: str
    user_agent: str
    ip_address: str

    def __init__(self):
        super().__init__(ChannelType.BROWSER)


# Completion
class Completion:
    type: CompletionInfoType

    def __init__(self, type_p: CompletionInfoType):
        self.type = type_p


class HostedCompletionInfo(Completion):
    callback_url: str
    success_url: str
    failure_url: bool

    def __init__(self):
        super().__init__(CompletionInfoType.HOSTED)


class NonHostedCompletionInfo(Completion):
    callback_url: str

    def __init__(self):
        super().__init__(CompletionInfoType.NON_HOSTED)


# Source
class SessionSource:
    type: SessionSourceType
    billing_address: SessionAddress
    home_phone: Phone
    mobile_phone: Phone
    work_phone: Phone
    email: str

    def __init__(self, type_p: SessionSourceType):
        self.type = type_p


class SessionCardSource(SessionSource):
    number: str
    expiry_month: int
    expiry_year: int
    name: str
    scheme: SessionScheme
    stored: bool
    store_for_future_use: bool

    def __init__(self):
        super().__init__(SessionSourceType.CARD)


class SessionIdSource(SessionSource):
    id: str

    def __init__(self):
        super().__init__(SessionSourceType.ID)


class SessionTokenSource(SessionSource):
    token: str
    store_for_future_use: bool

    def __init__(self):
        super().__init__(SessionSourceType.TOKEN)


class NetworkTokenSource(SessionSource):
    token: str
    expiry_month: int
    expiry_year: int
    name: str
    stored: bool

    def __init__(self):
        super().__init__(SessionSourceType.NETWORK_TOKEN)


class CardholderAccountInfo:
    purchase_count: int
    account_age: str
    add_card_attempts: int
    shipping_address_age: str
    account_name_matches_shipping_name: bool
    suspicious_account_activity: bool
    transactions_today: int
    authentication_method: AuthenticationMethod


class MerchantRiskInfo:
    delivery_email: str
    delivery_timeframe: DeliveryTimeframe
    is_preorder: bool
    is_reorder: bool
    shipping_indicator: ShippingIndicator


class Recurring:
    days_between_payments: int
    expiry: str


class Installment:
    number_of_payments: int
    days_between_payments: int
    expiry: str


class SessionRequest:
    source: SessionSource
    amount: int
    currency: Currency
    processing_channel_id: str
    marketplace: SessionMarketplaceData
    authentication_type: AuthenticationType
    authentication_category: Category
    account_info: CardholderAccountInfo
    challenge_indicator: ChallengeIndicator
    billing_descriptor: SessionsBillingDescriptor
    reference: str
    merchant_risk_info: MerchantRiskInfo
    prior_transaction_reference: str
    transaction_type: TransactionType
    shipping_address: SessionAddress
    shipping_address_matches_billing: bool
    completion: Completion
    channel_data: ChannelData
    recurring: Recurring
    installment: Installment


class ThreeDsMethodCompletionRequest:
    three_ds_method_completion: ThreeDsMethodCompletion



from datetime import datetime

from checkout_sdk.common.common import CustomerRequest
from checkout_sdk.common.common_four import MarketplaceData
from checkout_sdk.common.enums import Currency
from checkout_sdk.payments.payments import PaymentType, ShippingDetails, BillingInformation, PaymentRecipient, \
    ProcessingSettings, RiskRequest, ThreeDsRequest
from checkout_sdk.payments.payments_four import BillingDescriptor


class PaymentLinkRequest:
    payment_type: PaymentType.REGULAR
    amount: int
    currency: Currency
    payment_ip: str
    billing_descriptor: BillingDescriptor
    reference: str
    description: str
    expires_in: int
    customer: CustomerRequest
    shipping: ShippingDetails
    billing: BillingInformation
    recipient: PaymentRecipient
    processing: ProcessingSettings
    allow_payment_methods: list  # PaymentSourceType
    products: list  # Product
    metadata: dict
    three_ds: ThreeDsRequest
    risk: RiskRequest
    return_url: str
    locale: str
    capture: bool
    capture_on: datetime
    # Only available in Four
    processing_channel_id: str
    marketplace: MarketplaceData



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.common.enums import Currency
from checkout_sdk.files.files import FileRequest
from checkout_sdk.marketplace.marketplace import OnboardEntityRequest, MarketplacePaymentInstrument, \
    CreateTransferRequest, BalancesQuery, UpdateScheduleRequest


class MarketplaceClient(Client):
    __MARKETPLACE_PATH = 'marketplace'
    __INSTRUMENTS_PATH = 'instruments'
    __ENTITIES_PATH = 'entities'
    __FILES_PATH = 'files'
    __TRANSFERS_PATH = 'transfers'
    __BALANCES_PATH = 'balances'
    __PAYOUT_SCHEDULES_PATH = 'payout-schedules'

    def __init__(self, api_client: ApiClient,
                 files_client: ApiClient,
                 transfers_client: ApiClient,
                 balances_client: ApiClient,
                 configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.OAUTH)
        self.__files_client = files_client
        self.__transfers_client = transfers_client
        self.__balances_client = balances_client

    def create_entity(self, onboard_entity_request: OnboardEntityRequest):
        return self._api_client.post(self.build_path(self.__MARKETPLACE_PATH, self.__ENTITIES_PATH),
                                     self._sdk_authorization(), onboard_entity_request)

    def get_entity(self, entity_id: str):
        return self._api_client.get(self.build_path(self.__MARKETPLACE_PATH, self.__ENTITIES_PATH, entity_id),
                                    self._sdk_authorization())

    def update_entity(self, entity_id: str, onboard_entity_request: OnboardEntityRequest):
        return self._api_client.put(self.build_path(self.__MARKETPLACE_PATH, self.__ENTITIES_PATH, entity_id),
                                    self._sdk_authorization(), onboard_entity_request)

    def create_payment_instrument(self, entity_id: str, marketplace_payment_instrument: MarketplacePaymentInstrument):
        return self._api_client.post(
            self.build_path(self.__MARKETPLACE_PATH, self.__ENTITIES_PATH, entity_id, self.__INSTRUMENTS_PATH),
            self._sdk_authorization(), marketplace_payment_instrument)

    def upload_file(self, file_request: FileRequest):
        return self.__files_client.submit_file(self.__FILES_PATH, self._sdk_authorization(), file_request,
                                               multipart_file='path')

    def initiate_transfer_of_funds(self, create_transfer_request: CreateTransferRequest, idempotency_key: str = None):
        return self.__transfers_client.post(self.__TRANSFERS_PATH, self._sdk_authorization(), create_transfer_request,
                                            idempotency_key)

    def retrieve_a_transfer(self, transfer_id):
        return self.__transfers_client.get(self.build_path(self.__TRANSFERS_PATH, transfer_id),
                                           self._sdk_authorization())

    def retrieve_entity_balances(self, entity_id: str, balances_query: BalancesQuery):
        return self.__balances_client.get(self.build_path(self.__BALANCES_PATH, entity_id), self._sdk_authorization(),
                                          balances_query)

    def update_payout_schedule(self, entity_id: str, currency: Currency,
                               update_schedule_request: UpdateScheduleRequest):
        return self._api_client.put(
            self.build_path(self.__MARKETPLACE_PATH, self.__ENTITIES_PATH, entity_id, self.__PAYOUT_SCHEDULES_PATH),
            self._sdk_authorization(), {currency: update_schedule_request})

    def retrieve_payout_schedule(self, entity_id: str):
        return self._api_client.get(
            self.build_path(self.__MARKETPLACE_PATH, self.__ENTITIES_PATH, entity_id, self.__PAYOUT_SCHEDULES_PATH),
            self._sdk_authorization())



from enum import Enum

from checkout_sdk.common.common import Phone, Address
from checkout_sdk.common.common_four import ResidentialStatusType, AccountHolderIdentification
from checkout_sdk.common.enums import Currency, InstrumentType, Country
from checkout_sdk.common.enums_four import AccountType, AccountHolderType


class ContactDetails:
    phone: Phone


class Profile:
    urls: list
    mccs: list
    default_holding_currency: Currency


class EntityDocument:
    file_id: str
    type: str


class EntityIdentificationType(str, Enum):
    PASSPORT = 'passport'
    NATIONAL_IDENTITY_CARD = 'national_identity_card'
    DRIVING_LICENSE = 'driving_license'
    CITIZEN_CARD = 'citizen_card'
    RESIDENCE_PERMIT = 'residence_permit'
    ELECTORAL_ID = 'electoral_id'


class EntityIdentificationDocument:
    type: EntityIdentificationType
    front: str
    back: str


class EntityIdentification:
    national_id_number: str
    document: EntityIdentificationDocument


class EntityRepresentative:
    first_name: str
    middle_name: str
    last_name: str
    address: Address
    identification: EntityIdentification


class Company:
    business_registration_number: str
    legal_name: str
    trading_name: str
    principal_address: Address
    registered_address: Address
    document: EntityDocument
    representatives: list  # EntityRepresentative


class DateOfBirth:
    day: int
    month: int
    year: int


class Identification:
    national_id_number: str
    document: EntityIdentificationDocument


class Individual:
    first_name: str
    middle_name: str
    last_name: str
    trading_name: str
    national_tax_id: str
    registered_address: Address
    date_of_birth: DateOfBirth
    identification: Identification


class OnboardEntityRequest:
    reference: str
    contact_details: ContactDetails
    profile: Profile
    company: Company
    individual: Individual


class InstrumentDocument:
    type: str
    file_id: str


class BankDetails:
    name: str
    branch: str
    address: Address


class MarketplaceAccountHolder:
    type: AccountHolderType
    first_name: str
    last_name: str
    company_name: str
    tax_id: str
    date_of_birth: DateOfBirth
    country_of_birth: Country
    residential_status: ResidentialStatusType
    billing_address: Address
    phone: Phone
    identification: AccountHolderIdentification
    email: str


class MarketplacePaymentInstrument:
    type = InstrumentType.BANK_ACCOUNT
    label: str
    account_type: AccountType
    account_number: str
    bank_code: str
    branch_code: str
    iban: str
    bban: str
    swift_bic: str
    currency: Currency
    country: Country
    document: InstrumentDocument
    account_holder: MarketplaceAccountHolder
    bank: BankDetails


class ScheduleFrequency(str, Enum):
    WEEKLY = 'weekly'
    DAILY = 'daily'
    MONTHLY = 'monthly'


class DaySchedule(str, Enum):
    MONDAY = 'monday'
    TUESDAY = 'tuesday'
    WEDNESDAY = 'wednesday'
    THURSDAY = 'thursday'
    FRIDAY = 'friday'
    SATURDAY = 'saturday'
    SUNDAY = 'sunday'


class ScheduleRequest:
    frequency: ScheduleFrequency

    def __init__(self, frequency_p: ScheduleFrequency):
        self.frequency = frequency_p


class ScheduleFrequencyDailyRequest(ScheduleRequest):
    def __init__(self):
        super().__init__(ScheduleFrequency.DAILY)


class ScheduleFrequencyMonthlyRequest(ScheduleRequest):
    by_month_day: list  # int

    def __init__(self):
        super().__init__(ScheduleFrequency.MONTHLY)


class ScheduleFrequencyWeeklyRequest(ScheduleRequest):
    by_day: list  # DaySchedule

    def __init__(self):
        super().__init__(ScheduleFrequency.WEEKLY)


class UpdateScheduleRequest:
    enabled: bool
    threshold: int
    recurrence: ScheduleRequest


# Transfers

class TransferType(str, Enum):
    COMMISSION = 'commission'
    PROMOTION = 'promotion'
    REFUND = 'refund'


class TransferSource:
    id: str
    amount: int


class TransferDestination:
    id: str


class CreateTransferRequest:
    reference: str
    transfer_type: TransferType
    source: TransferSource
    destination: TransferDestination


# Balances

class BalancesQuery:
    query: str



from __future__ import absolute_import

from checkout_sdk.default_sdk import DefaultSdk  # noqa
from checkout_sdk.four.four_sdk import FourSdk  # noqa
from checkout_sdk.four.oauth_sdk import OAuthSdk  # noqa



from enum import Enum

from checkout_sdk.common.common import Address, Phone
from checkout_sdk.common.enums import Country
from checkout_sdk.common.enums_four import AccountHolderIdentificationType, AccountHolderType


class ResidentialStatusType(str, Enum):
    RESIDENT = 'resident'
    NON_RESIDENT = 'non_resident'


class AccountHolderIdentification:
    type: AccountHolderIdentificationType
    number: str
    issuing_country: Country


class AccountHolder:
    type: AccountHolderType
    first_name: str
    last_name: str
    company_name: str
    tax_id: str
    date_of_birth: str
    country_of_birth: Country
    residential_status: ResidentialStatusType
    billing_address: Address
    phone: Phone
    identification: AccountHolderIdentification
    email: str


class BankDetails:
    name: str
    branch: str
    address: Address


class UpdateCustomerRequest:
    id: str
    default: bool


class MarketplaceCommission:
    amount: int
    percentage: float


class MarketplaceDataSubEntity:
    id: str
    amount: int
    reference: str
    commission: MarketplaceCommission


class MarketplaceData:
    sub_entity_id: str
    sub_entities: list  # MarketplaceDataSubEntity


class Product:
    name: str
    quantity: int
    unit_price: int
    reference: str
    image_url: str
    url: str
    total_amount: int
    tax_amount: int
    discount_amount: int
    sku: str
    goods_id: str
    wxpay_goods_id: str



from checkout_sdk.common.enums import Currency


class QuoteRequest:
    source_currency: Currency
    source_amount: int
    destination_currency: Currency
    destination_amount: int
    process_channel_id: str



from __future__ import absolute_import

from checkout_sdk.common.common import Address
from checkout_sdk.common.enums import PaymentSourceType
from checkout_sdk.payments.payments_four import PaymentRequestSource


class RequestIdealSource(PaymentRequestSource):
    bic: str
    description: str
    language: str

    def __init__(self):
        super().__init__(PaymentSourceType.IDEAL)


class RequestSofortSource(PaymentRequestSource):

    def __init__(self):
        super().__init__(PaymentSourceType.SOFORT)


class RequestTamaraSource(PaymentRequestSource):
    billing_address: Address

    def __init__(self):
        super().__init__(PaymentSourceType.TAMARA)


class RequestPayPalSource(PaymentRequestSource):

    def __init__(self):
        super().__init__(PaymentSourceType.PAYPAL)


class PaymentRequestWeChatPaySource(PaymentRequestSource):
    billing_address: Address

    def __init__(self):
        super().__init__(PaymentSourceType.WECHATPAY)


class RequestAlipayPlusHKSource(PaymentRequestSource):

    def __init__(self):
        super().__init__(PaymentSourceType.ALIPAY_HK)


class RequestAlipayPlusCNSource(PaymentRequestSource):

    def __init__(self):
        super().__init__(PaymentSourceType.ALIPAY_CN)


class RequestAlipayPlusGCashSource(PaymentRequestSource):

    def __init__(self):
        super().__init__(PaymentSourceType.GCASH)



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.payments.payments import RefundRequest, VoidRequest
from checkout_sdk.payments.payments_four import PaymentRequest, PayoutRequest, CaptureRequest, AuthorizationRequest


class PaymentsClient(Client):
    __PAYMENTS_PATH = 'payments'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY_OR_OAUTH)

    def request_payment(self, payment_request: PaymentRequest, idempotency_key: str = None):
        return self._api_client.post(self.__PAYMENTS_PATH, self._sdk_authorization(), payment_request, idempotency_key)

    def request_payout(self, payout_request: PayoutRequest, idempotency_key: str = None):
        return self._api_client.post(self.__PAYMENTS_PATH, self._sdk_authorization(), payout_request, idempotency_key)

    def get_payment_details(self, payment_id: str):
        return self._api_client.get(self.build_path(self.__PAYMENTS_PATH, payment_id), self._sdk_authorization())

    def get_payment_actions(self, payment_id: str):
        return self._api_client.get(self.build_path(self.__PAYMENTS_PATH, payment_id, 'actions'),
                                    self._sdk_authorization())

    def capture_payment(self, payment_id: str, capture_request: CaptureRequest = None, idempotency_key: str = None):
        return self._api_client.post(self.build_path(self.__PAYMENTS_PATH, payment_id, 'captures'),
                                     self._sdk_authorization(), capture_request, idempotency_key)

    def refund_payment(self, payment_id: str, refund_request: RefundRequest = None, idempotency_key: str = None):
        return self._api_client.post(self.build_path(self.__PAYMENTS_PATH, payment_id, 'refunds'),
                                     self._sdk_authorization(), refund_request, idempotency_key)

    def void_payment(self, payment_id: str, void_request: VoidRequest = None, idempotency_key: str = None):
        return self._api_client.post(self.build_path(self.__PAYMENTS_PATH, payment_id, 'voids'),
                                     self._sdk_authorization(), void_request, idempotency_key)

    def increment_payment_authorization(self, payment_id: str, authorization_request: AuthorizationRequest = None,
                                        idempotency_key: str = None):
        return self._api_client.post(self.build_path(self.__PAYMENTS_PATH, payment_id, 'authorizations'),
                                     self._sdk_authorization(), authorization_request, idempotency_key)



from __future__ import absolute_import

from enum import Enum


class AccountType(str, Enum):
    SAVINGS = 'savings'
    CURRENT = 'current'
    CASH = 'cash'


class AccountHolderType(str, Enum):
    INDIVIDUAL = 'individual'
    CORPORATE = 'corporate'
    INSTRUMENT = 'instrument'


class AccountHolderIdentificationType(str, Enum):
    PASSPORT = 'passport'
    DRIVING_LICENSE = 'driving_licence'
    NATIONAL_ID = 'national_id'
    COMPANY_REGISTRATION = 'company_registration'
    TAX_ID = 'tax_id'



from checkout_sdk.common.enums import Country


class Address:
    address_line1: str
    address_line2: str
    city: str
    state: str
    zip: str
    country: Country


class Phone:
    country_code: str
    number: str


class CustomerRequest:
    id: str
    email: str
    name: str
    phone: Phone


class Product:
    name: str
    quantity: int
    price: int



from __future__ import absolute_import

from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.utils import map_to_http_metadata


class CheckoutException(Exception):

    def __init__(self, message=None):
        super().__init__(message)


class CheckoutArgumentException(CheckoutException):
    pass


class CheckoutAuthorizationException(CheckoutException):

    def __init__(self, message=None):
        super().__init__(message)

    @staticmethod
    def invalid_authorization(authorization_type: AuthorizationType):
        raise CheckoutAuthorizationException(
            'Operation requires ' + authorization_type.name + ' authorization type')

    @staticmethod
    def invalid_key(key_type: AuthorizationType):
        raise CheckoutAuthorizationException('%s is required for this operation' % key_type.name)


class CheckoutApiException(CheckoutException):
    http_metadata: dict
    error_details: dict

    def __init__(self, response):
        self.http_metadata = map_to_http_metadata(response)
        if response.text:
            payload = response.json()
            self.error_details = payload['error_codes'] if 'error_codes' in payload else None
        super().__init__('The API response status code ({}) does not indicate success.'
                         .format(response.status_code))



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.customers.customers_client_four import CustomersClient
from checkout_sdk.disputes.disputes_client import DisputesClient
from checkout_sdk.forex.forex_client import ForexClient
from checkout_sdk.four.checkout_four_apm_api import CheckoutApmApi
from checkout_sdk.instruments.instruments_four_client import InstrumentsClient
from checkout_sdk.marketplace.marketplace_client import MarketplaceClient
from checkout_sdk.payments.hosted.hosted_payments_client import HostedPaymentsClient
from checkout_sdk.payments.links.payments_client import PaymentsLinksClient
from checkout_sdk.payments.payments_client_four import PaymentsClient
from checkout_sdk.risk.risk_client import RiskClient
from checkout_sdk.sessions.sessions_client import SessionsClient
from checkout_sdk.tokens.tokens_client import TokensClient
from checkout_sdk.workflows.workflows_client import WorkflowsClient


def _base_api_client(configuration: CheckoutConfiguration) -> ApiClient:
    return ApiClient(configuration, configuration.environment.base_uri)


def _files_api_client(configuration: CheckoutConfiguration) -> ApiClient:
    return ApiClient(configuration, configuration.environment.files_uri)


def _transfers_api_client(configuration: CheckoutConfiguration) -> ApiClient:
    return ApiClient(configuration, configuration.environment.transfers_uri)


def _balances_api_client(configuration: CheckoutConfiguration) -> ApiClient:
    return ApiClient(configuration, configuration.environment.balances_uri)


class CheckoutApi(CheckoutApmApi):

    def __init__(self, configuration: CheckoutConfiguration):
        base_api_client = _base_api_client(configuration)
        super().__init__(base_api_client, configuration)
        self.tokens = TokensClient(api_client=base_api_client, configuration=configuration)
        self.customers = CustomersClient(api_client=base_api_client, configuration=configuration)
        self.instruments = InstrumentsClient(api_client=base_api_client, configuration=configuration)
        self.payments = PaymentsClient(api_client=base_api_client, configuration=configuration)
        self.sessions = SessionsClient(api_client=base_api_client, configuration=configuration)
        self.disputes = DisputesClient(api_client=base_api_client, configuration=configuration)
        self.forex = ForexClient(api_client=base_api_client, configuration=configuration)
        self.hosted_payments = HostedPaymentsClient(api_client=base_api_client, configuration=configuration)
        self.payments_links = PaymentsLinksClient(api_client=base_api_client, configuration=configuration)
        self.risk = RiskClient(api_client=base_api_client, configuration=configuration)
        self.workflows = WorkflowsClient(api_client=base_api_client, configuration=configuration)
        self.marketplace = MarketplaceClient(api_client=base_api_client,
                                             files_client=_files_api_client(configuration),
                                             transfers_client=_transfers_api_client(configuration),
                                             balances_client=_balances_api_client(configuration),
                                             configuration=configuration)



from __future__ import absolute_import

from abc import ABCMeta

from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.four.checkout_api import CheckoutApi
from checkout_sdk.four.four_keys_credentials import FourKeysSdkCredentials
from checkout_sdk.static_keys_builder import StaticKeysBuilder, validate_secret_key, validate_public_key


class FourStaticKeys(StaticKeysBuilder, metaclass=ABCMeta):
    _PUBLIC_KEY_PATTERN = '^pk_(sbox_)?[a-z2-7]{26}[a-z2-7*#$=]$'
    _SECRET_KEY_PATTERN = '^sk_(sbox_)?[a-z2-7]{26}[a-z2-7*#$=]$'

    def secret_key(self, secret_key):
        super().secret_key(secret_key)
        return self

    def public_key(self, public_key):
        super().public_key(public_key)
        return self


class FourSdk(FourStaticKeys):

    def __init__(self):
        super().__init__()

    def build(self):
        validate_secret_key(self._SECRET_KEY_PATTERN, self._secret_key)
        validate_public_key(self._PUBLIC_KEY_PATTERN, self._public_key)
        configuration = CheckoutConfiguration(
            credentials=FourKeysSdkCredentials(secret_key=self._secret_key, public_key=self._public_key),
            environment=self._environment,
            http_client=self._http_client)
        return CheckoutApi(configuration)



from __future__ import absolute_import

from datetime import datetime
from enum import Enum

import checkout_sdk
from checkout_sdk.common.common import CustomerRequest, Address, Phone
from checkout_sdk.common.common_four import AccountHolder, BankDetails, MarketplaceData
from checkout_sdk.common.enums import PaymentSourceType, Currency, Country
from checkout_sdk.common.enums_four import AccountType
from checkout_sdk.payments.payments import PaymentType, ShippingDetails, ThreeDsRequest, RiskRequest, \
    PaymentRecipient, ProcessingSettings, PaymentDestinationType


class AuthorizationType(str, Enum):
    FINAL = 'Final'
    ESTIMATED = 'Estimated'
    INCREMENTAL = 'Incremental'


class CaptureType(str, Enum):
    NON_FINAL = 'NonFinal'
    FINAL = 'Final'


class InstructionScheme(str, Enum):
    SWIFT = 'swift'
    LOCAL = 'local'
    INSTANT = 'instant'


class PaymentSenderType(str, Enum):
    INDIVIDUAL = 'individual'
    CORPORATE = 'corporate'
    INSTRUMENT = 'instrument'


class IdentificationType(str, Enum):
    PASSPORT = 'passport'
    DRIVING_LICENSE = 'driving_licence'
    NATIONAL_ID = 'national_id'


class PayoutSourceType(str, Enum):
    CURRENCY_ACCOUNT = 'currency_account'


class BillingDescriptor(checkout_sdk.payments.payments.BillingDescriptor):
    reference: str


class PaymentInstruction:
    purpose: str
    charge_bearer: str
    repair: bool
    scheme: InstructionScheme
    quote_id: str


class PayoutBillingDescriptor:
    reference: str


class Identification:
    type: IdentificationType
    number: str
    issuing_country: Country


# Payment Sender
class PaymentSender:
    type: PaymentSenderType

    def __init__(self, type_p: PaymentSenderType):
        self.type = type_p


class PaymentCorporateSender(PaymentSender):
    company_name: str
    address: Address

    def __init__(self):
        super().__init__(PaymentSenderType.CORPORATE)


class PaymentIndividualSender(PaymentSender):
    first_name: str
    last_name: str
    address: Address
    identification: Identification

    def __init__(self):
        super().__init__(PaymentSenderType.INDIVIDUAL)


class PaymentInstrumentSender(PaymentSender):

    def __init__(self):
        super().__init__(PaymentSenderType.INSTRUMENT)


# Payment Request Source
class PaymentRequestSource:
    type: PaymentSourceType

    def __init__(self, type_p: PaymentSourceType):
        self.type = type_p


class PaymentRequestCardSource(PaymentRequestSource):
    number: str
    expiry_month: int
    expiry_year: int
    name: str
    cvv: str
    stored: bool
    store_for_future_use: bool
    billing_address: Address
    phone: Phone

    def __init__(self):
        super().__init__(PaymentSourceType.CARD)


class PaymentRequestTokenSource(PaymentRequestSource):
    token: str
    billing_address: Address
    phone: Phone
    store_for_future_use: bool

    def __init__(self):
        super().__init__(PaymentSourceType.TOKEN)


class PaymentRequestNetworkTokenSource(PaymentRequestSource):
    token: str
    expiry_month: int
    expiry_year: int
    token_type: str
    cryptogram: str
    eci: str
    stored: bool
    name: str
    cvv: str
    billing_address: Address
    phone: Phone

    def __init__(self):
        super().__init__(PaymentSourceType.NETWORK_TOKEN)


class PaymentRequestIdSource(PaymentRequestSource):
    id: str
    cvv: str

    def __init__(self):
        super().__init__(PaymentSourceType.ID)


class RequestProviderTokenSource(PaymentRequestSource):
    payment_method: str
    token: str
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.PROVIDER_TOKEN)


class RequestBankAccountSource(PaymentRequestSource):
    payment_method: str
    account_type: str
    country: Country
    account_number: str
    bank_code: str
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.BANK_ACCOUNT)


# Request Payment
class PaymentRequest:
    source: PaymentRequestSource
    amount: int
    currency: Currency
    payment_type: PaymentType
    merchant_initiated: bool
    reference: str
    description: str
    authorization_type: AuthorizationType
    capture: bool
    capture_on: datetime
    customer: CustomerRequest
    billing_descriptor: BillingDescriptor
    shipping: ShippingDetails
    three_ds: ThreeDsRequest
    processing_channel_id: str
    previous_payment_id: str
    risk: RiskRequest
    success_url: str
    failure_url: str
    payment_ip: str
    sender: PaymentSender
    recipient: PaymentRecipient
    marketplace: MarketplaceData
    processing: ProcessingSettings
    metadata: dict
    items: list  # four/Product


# Payout Request Source
class PayoutRequestSource:
    type: PayoutSourceType

    def __init__(self, type_p: PayoutSourceType):
        self.type = type_p


class PayoutRequestCurrencyAccountSource(PayoutRequestSource):
    id: str

    def __init__(self):
        super().__init__(PayoutSourceType.CURRENCY_ACCOUNT)


# Payment Request Destination
class PaymentRequestDestination:
    type: PaymentDestinationType

    def __init__(self, type_p: PaymentDestinationType):
        self.type = type_p


class PaymentBankAccountDestination(PaymentRequestDestination):
    account_type: AccountType
    account_number: str
    bank_code: str
    branch_code: str
    iban: str
    swift_bic: str
    country: Country
    account_holder: AccountHolder
    bank: BankDetails

    def __init__(self):
        super().__init__(PaymentDestinationType.BANK_ACCOUNT)


class PaymentRequestIdDestination(PaymentRequestDestination):
    id: str

    def __init__(self):
        super().__init__(PaymentDestinationType.ID)


# Request Payout
class PayoutRequest:
    source: PayoutRequestSource
    destination: PaymentRequestDestination
    amount: int
    currency: Currency
    reference: str
    billing_descriptor: PayoutBillingDescriptor
    sender: PaymentSender
    instruction: PaymentInstruction
    processing_channel_id: str


# Captures
class CaptureRequest:
    amount: int
    capture_type: CaptureType
    reference: str
    metadata: dict


# Authorization
class AuthorizationRequest:
    amount: int
    reference: str
    metadata: dict



from __future__ import absolute_import

from datetime import datetime
from enum import Enum

from checkout_sdk.common.common import Address, Phone, CustomerRequest
from checkout_sdk.common.enums import Currency, PaymentSourceType, ChallengeIndicator, Country


class Exemption(str, Enum):
    LOW_VALUE = 'low_value'
    SECURE_CORPORATE_PAYMENT = 'secure_corporate_payment'
    TRUSTED_LISTING = 'trusted_listing'
    TRANSACTION_RISK_ASSESSMENT = 'transaction_risk_assessment'
    THREE_DS_OUTAGE = '3ds_outage'
    SCA_DELEGATION = 'sca_delegation'
    OUT_OF_SCA_SCOPE = 'out_of_sca_scope'
    OTHER = 'other'
    LOW_RISK_PROGRAM = 'low_risk_program'


class PaymentDestinationType(str, Enum):
    BANK_ACCOUNT = 'bank_account'
    CARD = 'card'
    ID = 'id'
    TOKEN = 'token'


class PaymentType(str, Enum):
    REGULAR = 'Regular'
    RECURRING = 'Recurring'
    MOTO = 'MOTO'
    INSTALLMENT = 'Installment'


class Purpose(str, Enum):
    DONATIONS = 'donations'
    EDUCATION = 'education'
    EMERGENCY_NEED = 'emergency_need'
    EXPATRIATION = 'expatriation'
    FAMILY_SUPPORT = 'family_support'
    GIFTS = 'gifts'
    INCOME = 'income'
    INSURANCE = 'insurance'
    INVESTMENT = 'investment'
    IT_SERVICES = 'it_services'
    LEISURE = 'leisure'
    LOAN_PAYMENT = 'loan_payment'
    OTHER = 'other'
    PENSION = 'pension'
    ROYALTIES = 'royalties'
    SAVINGS = 'savings'
    TRAVEL_AND_TOURISM = 'travel_and_tourism'
    FINANCIAL_SERVICES = 'financial_services'
    MEDICAL_TREATMENT = 'medical_treatment'


class FundTransferType(str, Enum):
    AA = 'AA'
    PP = 'PP'
    FT = 'FT'
    FD = 'FD'
    PD = 'PD'
    LO = 'LO'
    OG = 'OG'


class PreferredSchema(str, Enum):
    VISA = 'visa'
    MASTERCARD = 'mastercard'
    CARTES_BANCAIRES = 'cartes_bancaires'


class ProductType(str, Enum):
    QR_CODE = 'QR Code'
    IN_APP = 'In-App'
    OFFICIAL_ACCOUNT = 'Official Account'
    MINI_PROGRAM = 'Mini Program'


class ThreeDSFlowType(str, Enum):
    CHALLENGED = 'challenged'
    FRICTIONLESS = 'frictionless'
    FRICTIONLESS_DELEGATED = 'frictionless_delegated'


class ShippingPreference(str, Enum):
    NO_SHIPPING = 'NO_SHIPPING'
    SET_PROVIDED_ADDRESS = 'SET_PROVIDED_ADDRESS'
    GET_FROM_FILE = 'GET_FROM_FILE'


class UserAction(str, Enum):
    PAY_NOW = 'PAY_NOW'
    CONTINUE = 'CONTINUE'


class MerchantInitiatedReason(str, Enum):
    DELAYED_CHARGE = 'Delayed_charge'
    RESUBMISSION = 'Resubmission'
    NO_SHOW = 'No_show'
    REAUTHORIZATION = 'Reauthorization'


class TerminalType(str, Enum):
    APP = 'APP'
    WAP = 'WAP'
    WEB = 'WEB'


class OsType(str, Enum):
    ANDROID = 'ANDROID'
    IOS = 'IOS'


class BillingDescriptor:
    name: str
    city: str


class BillingInformation:
    address: Address
    phone: Phone


class PaymentRecipient:
    dob: str
    account_number: str
    zip: str
    last_name: str


class ShippingDetails:
    address: Address
    phone: Phone


class RiskRequest:
    enabled: bool


class ThreeDsRequest:
    enabled: bool
    attempt_n3d: bool
    eci: str
    cryptogram: str
    xid: str
    version: str
    exemption: Exemption
    challenge_indicator: ChallengeIndicator
    # Only available in Four
    status: str
    authentication_date: datetime
    authentication_amount: int
    flow_type: ThreeDSFlowType
    status_reason_code: str
    challenge_cancel_reason: str
    score: str
    cryptogram_algorithm: str


class Payer:
    name: str
    email: str
    document: str


class Installments:
    count: str


class DLocalProcessingSettings:
    country: Country
    payer: Payer
    installments: Installments


class AirlineTicket:
    number: str
    issue_date: str
    issuing_carrier_code: str
    travel_agency_name: str
    travel_agency_code: str


class AirlinePassengerName:
    full_name: str


class AirlinePassenger:
    name: AirlinePassengerName
    date_of_birth: str
    country_code: Country


class AirlineFlightLegDetails:
    flight_number: int
    carrier_code: str
    service_class: str
    departure_date: str
    departure_time: str
    departure_airport: str
    arrival_airport: str
    stopover_code: str
    fare_basis_code: str


class AirlineData:
    ticket: AirlineTicket
    passenger: AirlinePassenger
    flight_leg_details: list  # AirlineFlightLegDetails


class ProcessingSettings:
    aft: bool
    merchant_initiated_reason: MerchantInitiatedReason
    dlocal: DLocalProcessingSettings
    # Only available on Four
    tax_amount: int
    shipping_amount: int
    preferred_scheme: PreferredSchema
    product_type: ProductType
    open_id: str
    original_order_amount: int
    receipt_id: str
    terminal_type: TerminalType
    os_type: OsType
    invoice_id: str
    brand_name: str
    locale: str
    shipping_preference: ShippingPreference
    user_action: UserAction
    set_transaction_context: list  # dict
    airline_data: list  # AirlineData


# Request Source
class RequestSource:
    type: PaymentSourceType

    def __init__(self, type_p: PaymentSourceType):
        self.type = type_p


class RequestCardSource(RequestSource):
    number: str
    expiry_month: int
    expiry_year: int
    name: str
    cvv: str
    stored: bool
    store_for_future_use: bool
    billing_address: Address
    phone: Phone

    def __init__(self):
        super().__init__(PaymentSourceType.CARD)


class RequestCustomerSource(RequestSource):
    id: str

    def __init__(self):
        super().__init__(PaymentSourceType.ID)


class RequestDLocalSource(RequestSource):
    number: str
    expiry_month: int
    expiry_year: int
    name: str
    cvv: str
    stored: bool
    billing_address: Address
    phone: Phone

    def __init__(self):
        super().__init__(PaymentSourceType.D_LOCAL)


class RequestIdSource(RequestSource):
    id: str
    cvv: str

    def __init__(self):
        super().__init__(PaymentSourceType.ID)


class RequestNetworkTokenSource(RequestSource):
    token: str
    expiry_month: int
    expiry_year: int
    token_type: str
    cryptogram: str
    eci: str
    stored: bool
    name: str
    cvv: str
    billing_address: Address
    phone: Phone

    def __init__(self):
        super().__init__(PaymentSourceType.NETWORK_TOKEN)


class RequestTokenSource(RequestSource):
    token: str
    billing_address: Address
    phone: Phone
    store_for_future_use: bool

    def __init__(self):
        super().__init__(PaymentSourceType.TOKEN)


# Request Destination
class RequestDestination:
    type: PaymentDestinationType
    first_name: str
    last_name: str

    def __init__(self, type_p: PaymentDestinationType):
        self.type = type_p


class PaymentRequestCardDestination(RequestDestination):
    number: str
    expiry_month: int
    expiry_year: int
    name: str
    billing_address: Address
    phone: Phone

    def __init__(self):
        super().__init__(PaymentDestinationType.CARD)


class PaymentRequestIdDestination(RequestDestination):
    id: str

    def __init__(self):
        super().__init__(PaymentDestinationType.ID)


class PaymentRequestTokenDestination(RequestDestination):
    token: str
    billing_address: Address
    phone: Phone

    def __init__(self):
        super().__init__(PaymentDestinationType.TOKEN)


# Request
class PaymentRequest:
    source: RequestSource
    amount: int
    currency: Currency
    payment_type: PaymentType
    merchant_initiated: bool
    reference: str
    description: str
    capture: bool
    capture_on: datetime
    customer: CustomerRequest
    billing_descriptor: BillingDescriptor
    shipping: ShippingDetails
    previous_payment_id: str
    risk: RiskRequest
    success_url: str
    failure_url: str
    payment_ip: str
    three_ds: ThreeDsRequest
    recipient: PaymentRecipient
    metadata: dict
    processing: ProcessingSettings


class PayoutRequest:
    destination: RequestDestination
    amount: int
    fund_transfer_type: FundTransferType
    currency: Currency
    payment_type: PaymentType
    reference: str
    description: str
    capture: bool
    capture_on: datetime
    customer: CustomerRequest
    billing_descriptor: BillingDescriptor
    shipping: ShippingDetails
    three_ds: ThreeDsRequest
    previous_payment_id: str
    risk: RiskRequest
    success_url: str
    failure_url: str
    payment_ip: str
    recipient: PaymentRecipient
    metadata: dict
    processing: dict


# Captures
class CaptureRequest:
    amount: int
    reference: str
    metadata: dict


# Refunds
class RefundRequest:
    amount: int
    reference: str
    metadata: dict


# Voids
class VoidRequest:
    reference: str
    metadata: dict



from datetime import datetime


class RetrieveEventsRequest:
    payment_id: str
    charge_id: str
    track_id: str
    reference: str
    skip: int
    limit: int
    from_: datetime
    to: datetime
